program main

  use vtype_module
  use const_module, only : grav, const_set
  use ni3_module
  use netcdf

  implicit none

  interface
    subroutine findfu( fn, ios, fu, mode )
      use vtype_module
      implicit none
      character(len=*), intent(in )           :: fn
      integer(i4b)    , intent(out)           :: ios, fu
      character(len=*), intent(in ), optional :: mode
    end subroutine findfu
  end interface

  integer                          :: daydiv = 24

  character(len=extstr)            :: mode

  character(len=extstr)            :: ncfn_in
  character(len=extstr)            :: ncfn_ls
  character(len=extstr)            :: ncfn_ps
  character(len=extstr)            :: ncfn_out
  integer                          :: ncid_in
  integer                          :: ncid_ls
  integer                          :: ncid_ps
  integer                          :: ncid_out
  character(len=extstr)            :: varname_in
  character(len=extstr)            :: varname_ls
  character(len=extstr)            :: varname_ps

  real(dp)                         :: RefPress

  character(len=extstr)            :: name
  character(len=extstr)            :: stdname
  character(len=extstr)            :: units
  integer(i4b)         , parameter :: ndims = 3
  character(len=extstr)            :: dimname( ndims )

  integer                          :: imax
  integer                          :: jmax
  integer                          :: kmax
  integer                          :: tmax

  real(dp)           , allocatable :: x_Lon(:)
  real(dp)           , allocatable :: y_Lat(:)
  real(dp)           , allocatable :: xyr_Arr (:,:,:)
  real(dp)           , allocatable :: xy_Arr  (:,:)
  real(dp)           , allocatable :: xy_Ps   (:,:)
  real(dp)           , allocatable :: xy_DMArr(:,:)
  real(dp)           , allocatable :: a_Time(:)


  real(dp)                         :: Time
  real(dp)                         :: Ls
  real(dp)                         :: DMTime
  real(dp)                         :: DMLs
  real(dp)                         :: PrLs

  real(dp)                         :: FillValue

  logical                          :: ex
  integer                          :: ios

  integer                          :: DayStart
  integer                          :: DayEnd


  integer                          :: t, t1, t2
  integer                          :: t1s
  integer                          :: t1e


  character(extstr)                :: fn

  character(extstr)                :: ctlfn = "mkncfile.cntl"
  integer                          :: ctlfu

  namelist /input/   ncfn_in, varname_in, ncfn_ps, varname_ps, ncfn_ls, varname_ls
  namelist /press/   RefPress
  namelist /output/  ncfn_out
  namelist /dayinfo/ DayStart, DayEnd

  ncfn_in     = "ps"
  varname_in  = "Ps"
  ncfn_ls     = "ls"
  varname_ls  = "Ls"
  ncfn_out    = "out.nc"

  RefPress    = 700.0d0

!      dir    = "."
!      outfn  = "./dmps.dat"
!      landersym = "vl1"
!      landersym = "vl2"


  fn = ctlfn
  inquire( file = fn, exist = ex )
  if( .not. ex ) then
    write( 6, * ) "Control file ", trim( ctlfn ), " does not exit."
    write( 6, * ) "Control file should contain namelists listed below."
    write( 6, * ) "    &input"
    write( 6, * ) "        dir       = '.'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &lander"
    write( 6, * ) "        LanderSym = 'VL1'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &output"
    write( 6, * ) "        outfn     = 'vl1_dmps.dat'"
    write( 6, * ) "    &end"
    write( 6, * ) "    &dayinfo"
    write( 6, * ) "        DayStart  =     1"
    write( 6, * ) "        DayEnd    = 10000"
    write( 6, * ) "    &end"
    stop
  end if
  call findfu( fn, ios, ctlfu )
  if( ios /= 0 ) then
    write( 6, * ) 'STOP: ', ios
    stop
  end if
  open( ctlfu, file = fn, status = 'unknown' )
  read( ctlfu, input )
  rewind( ctlfu )
  read( ctlfu, output )
  rewind( ctlfu )
  read( ctlfu, press )
  rewind( ctlfu )
  read( ctlfu, dayinfo )
  close( ctlfu )


  call const_set( 1 )



  !----------------------------------------------------------
  !
  ! Preparing for ps file
  !
  mode = 'read'
  call ni3_open( ncfn_in, mode, ncid_in )
  call ni3_open( ncfn_ls, mode, ncid_ls )
  call ni3_open( ncfn_ps, mode, ncid_ps )
  !
  !----------------------------------------------------------

  call ni3_inq_dimlen( ncid_in, 'lon' , imax )
  call ni3_inq_dimlen( ncid_in, 'lat' , jmax )
  call ni3_inq_dimlen( ncid_in, 'sig' , kmax )
  call ni3_inq_dimlen( ncid_in, 'time', tmax )

  allocate( x_Lon (0:imax-1) )
  allocate( y_Lat (1:jmax  ) )
  allocate( a_Time(1:tmax  ) )

  call ni3_get_var( ncid_in, 'lon' , x_Lon  )
  call ni3_get_var( ncid_in, 'lat' , y_Lat  )
  call ni3_get_var( ncid_in, 'time', a_Time )


  allocate( xyr_Arr (0:imax-1, 1:jmax, 0:kmax) )
  allocate( xy_Arr  (0:imax-1, 1:jmax) )
  allocate( xy_Ps   (0:imax-1, 1:jmax) )
  allocate( xy_DMArr(0:imax-1, 1:jmax) )

  !----------------------------------------------------------
  !
  ! Preparing for output file
  !
  mode = 'new'
  call ni3_open( ncfn_out, mode, ncid_out )

  name    = "lon"
  stdname = "longitude"
  units   = "degrees_east"
  call ni3_set_dim( ncid_out, name, NF90_REAL, x_Lon , stdname = stdname, units  = units )
  call ni3_cp_atts( ncid_in, ncid_out, name )

  name    = "lat"
  stdname = "latitude"
  units   = "degrees_north"
  call ni3_set_dim( ncid_out, name, NF90_REAL, y_Lat , stdname = stdname, units  = units )
  call ni3_cp_atts( ncid_in, ncid_out, name )

!!$  name    = "time"
!!$  stdname = "time"
!!$  units   = "days since 0000-01-01"
!!$  call ni3_def_dim( ncid_out, name, NF90_REAL, NF90_UNLIMITED, stdname = stdname, units  = units )
!!$  call ni3_cp_atts( ncid_ice, ncid_out, name )

  name    = varname_ls
  stdname = varname_ls
  units   = "degrees"
  call ni3_def_dim( ncid_out, name, NF90_REAL, NF90_UNLIMITED, stdname = stdname, units  = units )
  call ni3_cp_atts( ncid_ls, ncid_out, name )

  dimname( 1 ) = "lon"
  dimname( 2 ) = "lat"
!!$  dimname( 3 ) = "time"
  dimname( 3 ) = varname_ls

  name         = varname_in
  stdname      = varname_in
  units        = "1"
  call ni3_def_var( ncid_out, name, NF90_REAL, ndims, dimname, stdname = stdname, units = units )
  call ni3_cp_atts( ncid_in, ncid_out, name )

  if ( ni3_chk_att( ncid_in, varname_in, "missing_value" ) ) then
    call ni3_get_att( ncid_in , varname_in, "missing_value", FillValue )
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  else if ( ni3_chk_att( ncid_in, varname_in, "_FillValue" ) ) then
    call ni3_get_att( ncid_in , varname_in, "_FillValue"   , FillValue )
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  else
    FillValue = -999.0
    call ni3_put_att( ncid_out, varname_in, "missing_value", real(FillValue) )
    call ni3_put_att( ncid_out, varname_in, "_FillValue"   , real(FillValue) )
  end if

  !
  !----------------------------------------------------------


  write( 6, * ) "---"

  PrLs     = -1.0d100

!!$  time_loop: do t1 = 1, (tmax-1)/daydiv
  t1s = max( DayStart, 1 )
  t1e = min( DayEnd, (tmax-1)/daydiv )
  time_loop: do t1 = t1s, t1e

    print *, "Day ", t1

    xy_DMArr =  0.0d0
    DMTime   =  0.0d0
    DMLs     =  0.0d0
!!$    PrLs     = -1.0d100

    do t2 = 1, daydiv
      ! A lat '+1' means that an initial condition is stored in a file.
      t = ( t1 - 1 ) * daydiv + t2 + 1

      !
      ! Read ps data
      !
      call ni3_get_varss( ncid_in, varname_in, t, xyr_Arr )
      xy_Arr = xyr_Arr(:,:,0)
!!$      if( t2 == 1 ) write( 6, * ) xy_Arr(0,jmax/2+1)

      call ni3_get_varss( ncid_ps, varname_ps, t, xy_Ps )

      xy_DMArr = xy_DMArr + xy_Arr * RefPress / xy_Ps

      !
      ! Read date data
      !
      call ni3_get_varss( ncid_ls, varname_ls, t, Ls )

      time = a_Time(t)

!!$      if( Ls < PrLs ) Ls = Ls + 360.0d0
      if( Ls < PrLs )then
        do
          Ls = Ls + 360.0d0
          if( Ls > PrLs ) exit
        end do
      end if


      DMTime = DMTime + time
      DMLs   = DMLs   + Ls
      PrLs   = Ls

    end do


    xy_DMArr = xy_DMArr / dble( daydiv )
    DMTime   = DMTime   / dble( daydiv )
    DMLs     = DMLs     / dble( daydiv )

!!$    DMLs     = mod( DMLs, 360.0d0 )


!!$    if( ( t1 >= DayStart ) .and. ( t1 <= DayEnd ) ) then
!!$      call ni3_put_varss( ncid_out, "time", t1 - DayStart + 1, DMTime )
      call ni3_put_varss( ncid_out, varname_ls, t1 - DayStart + 1, DMLs     )
      call ni3_put_varss( ncid_out, varname_in, t1 - DayStart + 1, xy_DMArr )
!!$    else
!!$      print *, " --- No output"
!!$    end if


  end do time_loop


  call ni3_close( ncid_in  )
  call ni3_close( ncid_ls  )

  call ni3_close( ncid_out )


end program main
